@@ -106,7 +106,7 @@ $utf8 = 1;
         else
         { $win_langs{$i + 0x400} = $ms_langids[$i][0]; }
     }
-    %langs_win = map {$win_langs{$_} => $_} keys %win_langs;
+    %langs_win = map {my ($t) = $win_langs{$_}; my (@res) = ($t => $_); push (@res, $t => $_) if ($t =~ s/-.*$//o && ($_ & 0xFC00) == 0x400); @res} keys %win_langs;
     $i = 0;
     %langs_mac = map {$_ => $i++} @mac_langs;
 }
@@ -124,10 +124,11 @@ Reads all the names into memory
 sub read
 {
     my ($self) = @_;
+    $self->SUPER::read or return $self;
+
     my ($fh) = $self->{' INFILE'};
     my ($dat, $num, $stroff, $i, $pid, $eid, $lid, $nid, $len, $off, $here);
 
-    $self->SUPER::read or return $self;
     $fh->read($dat, 6);
     ($num, $stroff) = unpack("x2nn", $dat);
     for ($i = 0; $i < $num; $i++)
@@ -142,7 +143,7 @@ sub read
         {
             if ($pid == 1 && defined $apple_encodings[0][$eid])
             { $dat = TTF_word_utf8(pack("n*", map({$apple_encodings[0][$eid][$_]} unpack("C*", $dat)))); }
-            elsif ($pid == 2 && $eid == 2 && defined @cp_1252)
+            elsif ($pid == 2 && $eid == 2 && @cp_1252)
             { $dat = TTF_word_utf8(pack("n*", map({$cp_1252[0][$_]} unpack("C*", $dat)))); }
             elsif ($pid == 0 || $pid == 3 || ($pid == 2 && $eid == 1))
             { $dat = TTF_word_utf8($dat); }
@@ -185,7 +186,7 @@ sub out
                         { $str_trans = pack("C*",
                                 map({$apple_encodings[1][$eid]{$_} || 0x3F} unpack("n*",
                                 TTF_utf8_word($str_trans)))); }
-                        elsif ($pid == 2 && $eid == 2 && defined @cp_1252)
+                        elsif ($pid == 2 && $eid == 2 && @cp_1252)
                         { $str_trans = pack("C*",
                                 map({$cp_1252[1][$eid]{$_} || 0x3F} unpack("n*",
                                 TTF_utf8_word($str_trans)))); }
@@ -292,7 +293,7 @@ sub is_utf8
 {
     my ($self, $pid, $eid) = @_;
 
-    return ($utf8 && ($pid == 0 || $pid == 3 || ($pid == 2 && ($eid != 2 || defined @cp_1252))
+    return ($utf8 && ($pid == 0 || $pid == 3 || ($pid == 2 && ($eid != 2 || @cp_1252))
             || ($pid == 1 && defined $apple_encodings[$eid])));
 }
 
@@ -333,33 +334,81 @@ sub find_name
 }
 
 
-=head2 set_name($nid, $str, $lang)
+=head2 remove_name($nid)
+
+Removes all strings with the given name id from the table.
+
+=cut
+
+sub remove_name
+{
+    my ($self, $nid) = @_;
+
+    delete $self->{'strings'}[$nid];
+}
+
+=head2 set_name($nid, $str[, $lang[, @cover]])
 
 Sets the given name id string to $str for all platforms and encodings that
 this module can handle. If $lang is set, it is interpretted as a language
 tag and if the particular language of a string is found to match, then
 that string is changed, otherwise no change occurs.
 
-Notice that this function does not add any names to the table.
+If supplied, @cover should be a list of references to two-element arrays 
+containing pid,eid pairs that should be added to the name table if not already present.
+
+This function does not add any names to the table unless @cover is supplied. 
 
 =cut
 
 sub set_name
 {
-    my ($self, $nid, $str, $lang) = @_;
-    my ($pid, $eid, $lid);
+    my ($self, $nid, $str, $lang, @cover) = @_;
+    my ($pid, $eid, $lid, $c);
 
     foreach $pid (0 .. $#{$self->{'strings'}[$nid]})
     {
+        my $strNL = $str;
+        $strNL =~ s/(?:\r?)\n/\r\n/og  if $pid == 3;
+        $strNL =~ s/(?:\r?)\n/\r/og    if $pid == 1;
         foreach $eid (0 .. $#{$self->{'strings'}[$nid][$pid]})
         {
+            if (defined $self->{'strings'}[$nid][$pid][$eid])
+            {
+                my ($isincover) = 0;
+                foreach $c (@cover)
+                {
+                    if ($c->[0] == $pid && $c->[1] == $eid)
+                    {
+                        $isincover = 1;
+                        last;
+                    }
+                }
+                push(@cover, [$pid, $eid]) if (!$isincover);
+            }
             foreach $lid (keys %{$self->{'strings'}[$nid][$pid][$eid]})
             {
                 next unless (!defined $lang || $self->match_lang($pid, $lid, $lang));
-                $self->{'strings'}[$nid][$pid][$eid]{$lid} = $str;
+                $self->{'strings'}[$nid][$pid][$eid]{$lid} = $strNL;
+                foreach $c (0 .. $#cover)
+                {
+                    next unless (defined $cover[$c] && $cover[$c][0] == $pid && $cover[$c][1] == $eid);
+                    delete $cover[$c];
+                    last;
+                }
             }
         }
     }
+    foreach $c (@cover)
+    {
+        next unless (defined $c && scalar @$c);
+        my ($pid, $eid) = @{$c};
+        my ($lid) = $self->find_lang($pid, $lang);
+        my $strNL = $str;
+        $strNL =~ s/\n/\r\n/og  if $pid == 3;
+        $strNL =~ s/\n/\r/og    if $pid == 1;
+        $self->{'strings'}[$nid][$pid][$eid]{$lid} = $strNL;
+    }
     return $self;
 }
 
@@ -378,8 +427,9 @@ sub match_lang
     my ($self, $pid, $lid, $lang) = @_;
     my ($langid) = $self->get_lang($pid, $lid);
 
+    return 1 if ($pid == 0);        # all languages are equal in unicode since nothing defined
     return ($lid == $lang) if ($lang != 0 || $lang eq '0');
-    return !index(lc($langid), lc($lang));
+    return !index(lc($langid), lc($lang)) || !index(lc($lang), lc($langid));
 }
 
 =head2 Font::TTF::Name->get_lang($pid, $lid)
@@ -417,6 +467,42 @@ sub find_lang
     return undef;
 }
 
+=head2 Font::TTF::Name->pe_list()
+
+Returns an array of references to two-element arrays 
+containing pid,eid pairs that already exist in this name table.
+Useful for creating @cover parameter to set_name().
+
+=cut
+
+sub pe_list
+{
+	my ($self) = @_;
+	my (@cover, %ids);
+
+	foreach my $nid (0 .. $#{$self->{'strings'}})
+	{
+		if (defined $self->{'strings'}[$nid])
+		{
+    		foreach my $pid (0 .. $#{$self->{'strings'}[$nid]})
+    		{
+    			if (defined $self->{'strings'}[$nid][$pid])
+    			{
+    				foreach my $eid (0 .. $#{$self->{'strings'}[$nid][$pid]})
+	    			{
+	    				if (defined $self->{'strings'}[$nid][$pid][$eid] && !$ids{$pid}{$eid})
+	    				{
+	    					$ids{$pid}{$eid} = 1;
+	    					push @cover, [$pid, $eid];
+	    				}
+	    			}
+	    		}
+    		}
+    	}
+    }
+	return @cover;
+}
+
 
 BEGIN {
 @apple_encs = (
@@ -556,22 +642,25 @@ M7'3)95=<=<UU-]QTRVUWW'7/?0\\],AC3SSUS',OO/3*:V^\]<Y['WSTR6=?
 1?/7-=S_\],MO?_S]!Y==>0@`
 EOT
 );
-#'
 
-@ms_langids = ( [],
+
+@ms_langids = ( [""],
     ['ar', ["-SA", "-IQ", "-EG", "-LY", "-DZ", "-MA", "-TN", 
             "-OM", "-YE", "-SY", "-JO", "-LB", "-KW", "-AE",
-            "-BH", "-QA"]],
+            "-BH", "-QA", "-Ploc-SA", "-145"]],
     ['bg-BG'],
     ['ca-ES'],
-    ['zh', ['-TW', 'CN', '-HK', '-SG', '-MO']],
+    ['zh', ['-TW', '-CN', '-HK', '-SG', '-MO', "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "zh", "-Hant"]],
     ["cs-CZ"],
     ["da-DK"],
     ["de", ["-DE", "-CH", "-AT", "-LU", "-LI"]],
     ["el-GR"],
     ["en", ["-US", "-UK", "-AU", "-CA", "-NZ", "-IE", "-ZA",
             "-JM", "029", "-BZ", "-TT", "-ZW", "-PH", "-ID",
-            "-HK", "-IN", "-MY", "-SG"]],
+            "-HK", "-IN", "-MY", "-SG", "-AE", "-BH", "-EG",
+            "-JO", "-KW", "-TR", "-YE"]],
     ["es", ["-ES", "-MX", "-ES", "-GT", "-CR", "-PA", "-DO",
             "-VE", "-CO", "-PE", "-AR", "-EC", "-CL", "-UY",
             "-PY", "-BO", "-SV", "-HN", "-NI", "-PR", "-US"]],
@@ -587,13 +676,17 @@ EOT
     ["ja-JP"],
     ["ko-KR"],
     ["nl", ["-NL", "-BE"]],
-    ["no", ["-bok-NO", "-nyn-NO"]],
+    ["no", ["-bok-NO", "-nyn-NO", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "nn", "nb"]],
     ["pl-PL"],
     ["pt", ["-BR", "-PT"]],
     ["rm-CH"],
     ["ro", ["-RO", "_MD"]],
     ["ru-RU"],
-    ["hr", ["-HR", "-Latn-CS", "Cyrl-CS", "-BA", "", "-Latn-BA", "-Cyrl-BA"]],
+    ["hr", ["-HR", "-Latn-CS", "Cyrl-CS", "-BA", "", "-Latn-BA", "-Cyrl-BA", 
+            "", "". "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "bs-Cyrl", "bs-Latn", "sr-Cyrl", "sr-Latn", "", "bs", "sr"]],
     ["sk-SK"],
     ["sq-AL"],
     ["sv", ["-SE", "-FI"]],
@@ -608,13 +701,19 @@ EOT
     ["et-EE"],
     ["lv-LV"],
     ["lt-LT"],
-    ["tg-Cyrl-TJ"],
+    ["tg", ["-Cyrl-TJ", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "-Cyrl"]],
     ["fa-IR"],
     ["vi-VN"],
     ["hy-AM"],
-    ["az", ["-Latn-AZ", "-Cyrl-AZ"]],
+    ["az", ["-Latn-AZ", "-Cyrl-AZ", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "-Cyrl", "-Latn"]],
     ["eu-ES"],
-    ["wen". ["wen-DE", "dsb-DE"]],
+    ["wen". ["wen-DE", "dsb-DE", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "dsb"]],
     ["mk-MK"],
 # 0030
     ["st"],
@@ -629,7 +728,8 @@ EOT
     ["hi-IN"],
     ["mt"],
     ["se", ["-NO", "-SE", "-FI", "smj-NO", "smj-SE", "sma-NO", "sma-SE",
-            "", "smn-FI"]],
+            "", "smn-FI", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "smn", "sms", "smj"]],
     ["ga-IE"],
     ["yi"],
     ["ms", ["-MY", "-BN"]],
@@ -638,10 +738,14 @@ EOT
     ["ky-KG"],
     ["sw-KE"],
     ["tk-TM"],
-    ["uz", ["-Latn-UZ", "-Cyrl-UZ"]],
+    ["uz", ["-Latn-UZ", "-Cyrl-UZ", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "-Cyrl", "-Latn"]],
     ["tt-RU"],
     ["bn", ["-IN", "-BD"]],
-    ["pa", ["-IN", "-Arab-PK"]],
+    ["pa", ["-IN", "-Arab-PK", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "-Arab"]],
     ["gu-IN"],
     ["or-IN"],
     ["ta-IN"],
@@ -652,7 +756,9 @@ EOT
     ["mr-IN"],
     ["sa-IN"],
 # 0050
-    ["mn", ["-Cyrl-MN", "-Mong-CN"]],
+    ["mn", ["-Cyrl-MN", "-Mong-CN", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "-Cyrl", "-Mong"]],
     ["bo", ["-CN", "-BT"]],
     ["cy-GB"],
     ["km-KH"],
@@ -661,13 +767,19 @@ EOT
     ["gl-ES"],
     ["kok-IN"],
     ["mni"],
-    ["sd", ["-IN", "-PK"]],
+    ["sd", ["-IN", "-PK", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "-Arab"]],
     ["syr-SY"],
     ["si-LK"],
-    ["chr"],
-    ["iu", ["-Cans-CA", "-Latn-CA"]],
+    ["chr", ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "-Cher"]],
+    ["iu", ["-Cans-CA", "-Latn-CA", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "-Cans", "-Latn"]],
     ["am-ET"],
-    ["tmz", ["-Arab", "tmz-Latn-DZ"]],
+    ["tmz", ["-Arab", "-Latn-DZ", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "-Latn"]],
 # 0060
     ["ks"],
     ["ne", ["-NP", "-IN"]],
@@ -676,8 +788,10 @@ EOT
     ["fil-PH"],
     ["dv-MV"],
     ["bin-NG"],
-    ["fuv-NG"], 
-    ["ha-Latn-NG"],
+    ["fuv", ["-NG", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "ff-Latn"]], 
+    ["ha", ["-Latn-NG", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "-Latn"]],
     ["ibb-NG"],
     ["yo-NG"],
     ["quz", ["-BO", "-EC", "-PE"]],
@@ -704,10 +818,10 @@ EOT
     [""],           # (unassigned)
 # 0080
     ["ug-CN"],
-    [""],           # (unassigned)
+    ["mi-NZ"],
     ["oc-FR"],
+    ["co-FR"],
     ["gsw-FR"],
-    [""],           # (unassigned)
     ["sah-RU"],
     ["qut-GT"],
     ["rw-RW"],
@@ -716,7 +830,19 @@ EOT
     [""],           # (unassigned)
     [""],           # (unassigned)
     ["gbz-AF"],
+    [""],           # (unassigned)
+    [""],           # (unassigned)
+    [""],           # (unassigned)
+# 0090
+    [""],           # (unassigned)
+    ["gd-GB"],
+    ["ku", ["-Arab-IQ", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+            "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "-Arab"]],
+    ["quc-CO"],
 );
+# 0501 = qps-ploc
+# 05fe = qps-ploca
+# 09ff = qps-plocm
 
 @mac_langs = (
     'en', 'fr', 'de', 'it', 'nl', 'sv', 'es', 'da', 'pt', 'no',
